package de.ronnyfriedland

import de.ronnyfriedland.scheduler.enums.EntryState;
import de.ronnyfriedland.scheduler.enums.PlannedDay;

/**
 * Controller für den Scheduler
 * @author ronnyfriedland
 */
class SchedulerController {
	
	def messageSource
	
	def index = {
		redirect(action:'list')
	}
	
	/**
	 * Ermittelt alle geplanten Einträge und gibt diese gruppiert nach dem Tag zurück.
	 */
	def list = {
		def plannedEntries = [:]
		
		for(PlannedDay day : PlannedDay.values()) {
			def entries = PlannedEntry.withCriteria {
				entry {
					not {
						eq("state", EntryState.DELETED)
					}
				}
				eq("plannedOnDay", day)
			}
			plannedEntries.put(day, entries)
		}
		render(view:'list',model:['plannedEntries':plannedEntries])
	}
	
	/**
	 * Ermittelt alle nicht-gelöschten Einträge, welche an diesem Tag noch nicht geplant wurden.
	 */
	def entries = {
		def entries = []
		def day = params.day
		
		if(!day) {
			flash.error = messageSource.getMessage("scheduler.error.plannedentry.load", null, request.getLocale())
		} else {
			def criteria = Entry.createCriteria()
			entries = criteria.listDistinct {
				not{
					eq("state", EntryState.DELETED)
				}
				//or {
				//	isEmpty("plannedEntries")
				//	plannedEntries {
				//		ne("plannedOnDay", PlannedDay.valueOf(day))
				//	}
				//}
			}
		}
		render (template:"entries", model:['entries':entries,'day':day], plugin:'scheduler-plugin')
	}
	
	/**
	 * Plant einen Eintrag zu einem bestimmten Tag.
	 */
	def addPlannedEntry = {
		def id = params.id
		def day = params.day
		
		if(!id || !day) {
			flash.error = messageSource.getMessage("scheduler.error.plannedentry.save", null, request.getLocale())
		} else {
			if(!savePlannedEntry(Entry.findById(id), PlannedDay.valueOf(day))) {
				flash.error = messageSource.getMessage("scheduler.error.plannedentry.save", null, request.getLocale())
			}
		}
		redirect(action:'list')
	}
	
	/**
	 * Entfernt einen geplanten Eintrag.
	 */
	def removePlannedEntry = {
		def id = params.id
		if(id) {
			def plannedEntry = PlannedEntry.findById(id)
			plannedEntry.delete()
		}
		redirect(action:'list')
	}
	
	/**
	 * Entfernt einen Eintrag und seine Referenzen
	 */
	def removeEntry = {
		def id = params.id
		if(id) {
			def entry = Entry.findById(id)
			if(entry) {
				def plannedEntry = PlannedEntry.findByEntry(entry)
				if(plannedEntry) {
					plannedEntry.delete()
				}
				entry.state = EntryState.DELETED
				entry.save()
			}
		}
		redirect(action:'list')
	}
	
	/**
	 * Erzeugt einen neuen Eintrag und plant diesen für den ausgewählten Tag
	 */
	def newEntry = {
		def title = params.title
		def description = params.description
		def time = params.time
		def day = params.day
		
		if(!saveEntry(title, description, time, EntryState.ACTIVE, day)) {
			flash.error = messageSource.getMessage("scheduler.error.entry.save", null, request.getLocale())
		}
		
		redirect(action:'list')
	}
	
	/**
	 * Bricht eine Aktion ab. Action enthält keine Funktion, sondern leitet direkt weiter.
	 */
	def cancel = {
		redirect(action:'list')
	}
	
	/**
	 * Speichert einen neuen Eintrag und plant diesen, sofern ein Tag angegeben wird.
	 * @param title
	 * @param description
	 * @param time
	 * @param state
	 * @param day
	 * @return
	 */
	def saveEntry(def title, def description, def time, def state, def day) {
		def success = false
		def entry = new Entry()
		entry.title = title
		entry.description = description
		entry.state = EntryState.ACTIVE
		entry.time = time
		if(entry.validate()) {
			entry.save()
			success = true
			if(day) {
				success = savePlannedEntry(entry, PlannedDay.valueOf(day))				
			}
		}
		return success
	}
	
	/**
	 * Speichert einen geplanten Eintrag.
	 * @param entry
	 * @param day
	 * @return
	 */
	def savePlannedEntry(def entry, def day) {
		def success = false
		def plannedEntry = new PlannedEntry()
		plannedEntry.entry = entry
		plannedEntry.plannedOnDay = day
		if(plannedEntry.validate()) {
			plannedEntry.save()
			success = true			
		}
		return success
	}
}
